@@ -39,7 +39,7 @@ from archipelcore.archipelHookableEntity import TNHookableEntity
 from archipelcore.archipelPermissionCenter import TNArchipelPermissionCenter
 from archipelcore.archipelRosterQueryableEntity import TNRosterQueryableEntity
 from archipelcore.archipelTaggableEntity import TNTaggableEntity
-from archipelcore.utils import TNArchipelLogger, build_error_iq
+from archipelcore.utils import TNArchipelLogger, build_error_iq, get_default_gateway_interface
 
 import archipelcore.archipelPermissionCenter
 import archipelcore.pubsub
@@ -126,6 +126,7 @@ class TNArchipelEntity (object):
         self.entity_type            = "not-defined"
         self.plugins                = []
         self.is_unregistering       = False
+        self.use_avatar             = self.configuration.getboolean("GLOBAL", "use_avatar");
         self.permission_db_file     = "permissions.sqlite3"
         self.permission_admin_names = dict(map(lambda x: ("STATIC_%s" % x, x), self.configuration.get("GLOBAL", "archipel_root_admins").split()))
         self.permission_center      = TNArchipelPermissionCenter(root_admins=self.permission_admin_names)
@@ -149,6 +150,7 @@ class TNArchipelEntity (object):
             self.create_hook("HOOK_ARCHIPELENTITY_XMPP_DISCONNECTED")
             self.create_hook("HOOK_ARCHIPELENTITY_XMPP_AUTHENTICATED")
             self.create_hook("HOOK_ARCHIPELENTITY_PLUGIN_ALL_LOADED")
+            self.create_hook("HOOK_ARCHIPELENTITY_VCARD_READY")
 
             ## recover/create pubsub after connection
             self.register_hook("HOOK_ARCHIPELENTITY_XMPP_AUTHENTICATED", self.recover_pubsubs)
@@ -158,6 +160,12 @@ class TNArchipelEntity (object):
         ip_conf = self.configuration.get("GLOBAL", "machine_ip")
         if ip_conf == "auto":
             self.ipaddr = socket.gethostbyname(socket.gethostname())
+        elif ip_conf == "gateway_interface":
+            try:
+                self.ipaddr = get_default_gateway_interface()
+            except:
+                self.log.warning("Configuration set to 'gateway_interface' but unable to find IP. Falling back in auto mode")
+                self.ipaddr = socket.gethostbyname(socket.gethostname())
         else:
             self.ipaddr = ip_conf
 
@@ -371,6 +379,12 @@ class TNArchipelEntity (object):
         else:
             self.log.warning("Trying to disconnect, but not connected. Ignoring.")
 
+    def reset_loop_status(self):
+        """
+        Reset the loop_status value to restart (used from callback when disconnect envent is raised)
+        """
+        self.loop_status=ARCHIPEL_XMPP_LOOP_RESTART
+
     ### Pubsub
 
     def recover_pubsubs(self, origin, user_info, arguments):
@@ -476,6 +490,8 @@ class TNArchipelEntity (object):
         self.xmppclient.RegisterHandler('message', self.process_message, typ="chat")
         self.xmppclient.RegisterHandler('iq', self.process_permission_iq, ns=ARCHIPEL_NS_PERMISSIONS)
         self.xmppclient.RegisterHandler('iq', self.process_subscription_iq, ns=ARCHIPEL_NS_SUBSCRIPTION)
+        self.xmppclient.RegisterHandler('iq', self.process_ping_iq, typ="get", ns="urn:xmpp:ping")
+        self.xmppclient.RegisterDisconnectHandler(self.reset_loop_status)
         for plugin in self.plugins:
             self.log.info("PLUGIN: registering stanza handler for plugin %s" % plugin["info"]["identifier"])
             plugin["plugin"].register_handlers()
@@ -497,6 +513,7 @@ class TNArchipelEntity (object):
         self.xmppclient.UnregisterHandler('message', self.process_message, typ="chat")
         self.xmppclient.UnregisterHandler('iq', self.process_permission_iq, ns=ARCHIPEL_NS_PERMISSIONS)
         self.xmppclient.UnregisterHandler('iq', self.process_subscription_iq, ns=ARCHIPEL_NS_SUBSCRIPTION)
+        self.xmppclient.UnregisterHandler('iq', self.process_ping_iq, typ="get", ns="urn:xmpp:ping")
         for plugin in self.plugins:
             self.log.info("PLUGIN: unregistering stanza handler for plugin %s" % plugin["info"]["identifier"])
             plugin["plugin"].unregister_handlers()
@@ -563,6 +580,19 @@ class TNArchipelEntity (object):
             conn.send(reply)
             raise xmpp.protocol.NodeProcessed
 
+    def process_ping_iq(self, conn, iq):
+        """
+        check the iq, if we've got a ping then pong it. for more info
+        take a look at: http://xmpp.org/extensions/xep-0199.html
+        @type conn: xmpp.Dispatcher
+        @param conn: the instance of xmpp connection
+        @type iq: xmpp.Protocol.Iq
+        @param iq: the received IQ stanza
+        """
+        rep = iq.buildReply(typ="result")
+        conn.send(rep)
+        raise xmpp.protocol.NodeProcessed
+
     def iq_add_subscription(self, iq):
         """
         Add a JID in the entity roster.
@@ -600,6 +630,21 @@ class TNArchipelEntity (object):
 
     ### XMPP Utilities
 
+    def change_name(self, newname, publish=True):
+        """
+        Rename the current entity
+        @type newname: string
+        @param newname: the new name
+        @type publish: Boolean
+        @param publish: if False, you need to manually publish the vCard if you want to
+        """
+        if self.name == newname:
+            return
+        self.name = newname
+        if publish:
+            self.set_vcard()
+
+
     def change_presence(self, presence_show=None, presence_status=None, callback=None):
         """
         Change the presence of the entity.
@@ -610,9 +655,11 @@ class TNArchipelEntity (object):
         """
         self.log.info("status change: %s show:%s" % (presence_status, presence_show))
         pres = xmpp.Presence(status=presence_status, show=presence_show)
+        xmpp.dispatcher.ID += 1
+        pres.setID("%s-%d" % (self.jid.getNode(), xmpp.dispatcher.ID))
 
         def presence_callback(conn, resp):
-            self.xmppstatus     = presence_status
+            self.xmppstatus = presence_status
             self.xmppstatusshow = presence_show
             self.log.debug("PRESENCE : I just set change presence. The result is %s" % resp)
             if callback:
@@ -626,9 +673,7 @@ class TNArchipelEntity (object):
         @type presence_status: string
         @param presence_status: the value of the XMPP status
         """
-        self.xmppstatus = presence_status
-        pres = xmpp.Presence(status=self.xmppstatus, show=self.xmppstatusshow)
-        self.xmppclient.send(pres)
+        self.change_presence(self.xmppstatusshow, presence_status)
 
     def push_change(self, namespace, change, content_node=None):
         """
@@ -705,6 +750,8 @@ class TNArchipelEntity (object):
         """
         self.log.info("%s is subscribing to jid %s" % (str(self.jid), str(jid)))
         presence = xmpp.Presence(to=jid, typ='subscribe')
+        xmpp.dispatcher.ID += 1
+        presence.setID("%s-%d" % (self.jid.getNode(), xmpp.dispatcher.ID))
         if self.name:
             presence.addChild(name="nick", namespace="http://jabber.org/protocol/nick", payload=self.name)
         self.xmppclient.send(presence)
@@ -764,11 +811,17 @@ class TNArchipelEntity (object):
 
     ### VCARD management
 
+    def manage_vcard_hook(self, origin, user_info, parameters):
+        """
+        Hook to manage VCard.
+        """
+        self.manage_vcard()
+
     def manage_vcard(self):
         """
         Retrieve vCard from server.
         """
-        self.log.info("Asking for own vCard.")
+        self.log.info("VCARD: Asking for own vCard.")
         node_iq = xmpp.Iq(typ='get')
         node_iq.addChild(name="vCard", namespace="vcard-temp")
         self.xmppclient.SendAndCallForResponse(stanza=node_iq, func=self.did_receive_vcard)
@@ -778,37 +831,39 @@ class TNArchipelEntity (object):
         Callback of manage_vcard()
         """
         self.vCard = vcard.getTag("vCard")
-        if self.vCard and self.vCard.getTag("PHOTO"):
-            self.b64Avatar = self.vCard.getTag("PHOTO").getTag("BINVAL").getCDATA()
-        self.log.info("Own vcard retrieved")
-        self.set_vcard()
+
+        if self.vCard:
+            self.log.info("VCARD: I already have a vCard.")
+            if self.use_avatar and self.vCard.getTag("PHOTO"):
+                self.b64Avatar = self.vCard.getTag("PHOTO").getTag("BINVAL").getCDATA()
+            self.get_custom_vcard_information(self.vCard)
+        else:
+            self.log.info("VCARD: I don't have any vCard. Creating new one.")
+            self.set_vcard()
 
     def set_vcard(self, params={}):
         """
-        Allows to define a vCard type for the entry.
+        Allows to define a vCard t ype for the entry.
         set the vcard of the entity
         @type params: dict
         @param params: the parameters of the vCard
         """
         try:
-            self.log.info("vCard making started.")
-            node_iq     = xmpp.Iq(typ='set', xmlns=None)
-            payload     = []
-            type_node   = xmpp.Node(tag="ROLE")
+            self.log.info("VCARD: vCard making started.")
+            node_iq = xmpp.Iq(typ='set', xmlns=None)
+            payload = []
+            type_node = xmpp.Node(tag="ROLE")
             type_node.setData(self.entity_type)
             payload.append(type_node)
 
-            if hasattr(self, "vcard_infos"):
-                for key, value in self.vcard_infos.items():
-                    node = xmpp.Node(tag=key.upper())
-                    node.setData(value)
-                    payload.append(node)
+            self.set_custom_vcard_information(payload)
 
             if self.name:
                 name_node = xmpp.Node(tag="FN")
                 name_node.setData(self.name)
                 payload.append(name_node)
-            if self.configuration.getboolean("GLOBAL", "use_avatar"):
+
+            if self.use_avatar:
                 if not self.b64Avatar:
                     if params and params["filename"]:
                         self.b64avatar_from_filename(params["filename"])
@@ -820,13 +875,16 @@ class TNArchipelEntity (object):
                 node_photo_data.setData(self.b64Avatar)
                 node_photo = xmpp.Node(tag="PHOTO", payload=[node_photo_content_type, node_photo_data])
                 payload.append(node_photo)
+
             node_iq.addChild(name="vCard", payload=payload, namespace="vcard-temp")
-            ## updating internal representation of the vCard
+
+            # updating internal representation of the vCard
             self.vCard = node_iq.getTag("vCard")
+
             self.xmppclient.SendAndCallForResponse(stanza=node_iq, func=self.send_update_vcard)
-            self.log.info("vCard information sent with type: %s" % self.entity_type)
+            self.log.info("VCARD: vCard information sent with type: %s" % self.entity_type)
         except Exception as ex:
-            self.log.error("Error during setting vCard (set_vcard) using stanza: %s EXCEPTION IS: %s" % (str(node_iq), str(ex)))
+            self.log.error("VCARD: Error during setting vCard (set_vcard) using stanza: %s EXCEPTION IS: %s" % (str(node_iq), str(ex)))
 
     def send_update_vcard(self, conn, presence, photo_hash=None):
         """
@@ -841,13 +899,28 @@ class TNArchipelEntity (object):
         @param photo_hash: the SHA-1 hash of the photo that changes (optionnal)
         """
         node_presence = xmpp.Presence(status=self.xmppstatus, show=self.xmppstatusshow)
+        xmpp.dispatcher.ID += 1
+        node_presence.setID("%s-%d" % (self.jid.getNode(), xmpp.dispatcher.ID))
+
         if photo_hash:
             node_photo_sha1 = xmpp.Node(tag="photo")
             node_photo_sha1.setData(photo_hash)
         node_presence.addChild(name="x", namespace='vcard-temp:x:update')
         self.xmppclient.send(node_presence)
+        self.perform_hooks("HOOK_ARCHIPELENTITY_VCARD_READY")
         self.log.info("vCard update presence sent.")
 
+    def set_custom_vcard_information(self, vCard):
+        """
+        Overide this to set custom info in vCard
+        """
+        pass
+
+    def get_custom_vcard_information(self, vCard):
+        """
+        Override this to update anything relative to current vCard
+        """
+        pass
 
     ### Inband registration management
 
@@ -1177,6 +1250,8 @@ class TNArchipelEntity (object):
                     return
                 if self.loop_status == ARCHIPEL_XMPP_LOOP_ON:
                     if self.xmppclient.isConnected():
+                        if hasattr(self, "on_xmpp_loop_tick"):
+                            self.on_xmpp_loop_tick()
                         self.xmppclient.Process(3)
                 elif self.loop_status == ARCHIPEL_XMPP_LOOP_RESTART:
                     if self.xmppclient.isConnected():
@@ -1200,3 +1275,10 @@ class TNArchipelEntity (object):
 
         if self.xmppclient.isConnected():
             self.xmppclient.disconnect()
+
+    def on_xmpp_loop_tick(self):
+        """
+        override it in subclass if you want to do something
+        in each executiong of run loop
+        """
+        pass
